#ifndef txtinput_h
#define txtinput_h

/* These stream input routines are written so that simple
 * conditionals can be used:
 *
 *      if (readxint(&myint, stdin)) {
 *         do_error_recovery; normally_abort_to_somewhere;
 *      }
 *      else {
 *         do_normal_things; usually_much_longer_than_bad_case;
 *      }
 *
 * They allow overflow detection, and permit other routines to
 * detect the character that terminated a numerical field. No
 * string storage is required, thus there is no limitation on
 * the length of input fields.  For example, a number entered
 * with a string of 1000 leading zeroes will not annoy these.
 *
 * The numerical input routines *NEVER* absorb a terminal '\n'.
 * Thus a sequence such as:
 *
 *      err = readxint(&myint, stdin);
 *      flushln(stdin);
 *
 * will always consume complete lines.
 *
 * They are also re-entrant, subject to the limitations of file
 * systems.  e.g interrupting readxint(v, stdin) operation with
 * a call to readxwd(wd, stdin) would not be well defined, if
 * the same stdin is being used for both calls.  If ungetc is
 * interruptible the run-time system is broken.
 */

/*--------------------------------------------------------------
 * Skip all blanks on f.  At completion getc(f) will return
 * a non-blank character, which may be \n or EOF
 *
 * Skipblks returns the char that getc will next return, or EOF.
 */
int skipblks(FILE *f);

/*--------------------------------------------------------------
 * Skip all whitespace on f, including \n, \f, \v, \r.  At
 * completion getc(f) will return a non-blank character, which
 * may be EOF
 *
 * Skipblks returns the char that getc will next return, or EOF.
 */
int skipwhite(FILE *f);

/*--------------------------------------------------------------
 * Read an unsigned value.  Signal error for overflow or no
 * valid number found. Returns true for error, false for noerror
 *
 * Skip all leading whitespace on f.  At completion getc(f) will
 * return the character terminating the number, which may be \n
 * or EOF among others. Barring EOF it will NOT be a digit.  The
 * combination of error and the following getc returning \n
 * indicates that no numerical value was found on the line.
 *
 * If the user wants to skip all leading white space including
 * \n, \f, \v, \r, he should first call "skipwhite(f);"
 *
 * Peculiarity: This specifically forbids a leading '+' or '-'.
 * Peculiarity: This forbids overflow, unlike C unsigned usage.
 *              on overflow, UINT_MAX is returned.
 */
bool readxwd(unsigned int *wd, FILE *f);

/*--------------------------------------------------------------
 * Read a signed value.  Signal error for overflow or no valid
 * number found.  Returns true for error, false for noerror.  On
 * overflow either INT_MAX or INT_MIN is returned in *val.
 *
 * Skip all leading whitespace on f.  At completion getc(f) will
 * return the character terminating the number, which may be \n
 * or EOF among others. Barring EOF it will NOT be a digit.  The
 * combination of error and the following getc returning \n
 * indicates that no numerical value was found on the line.
 *
 * If the user wants to skip all leading white space including
 * \n, \f, \v, \r, he should first call "skipwhite(f);"
 *
 * Peculiarity: an isolated leading '+' or '-' NOT immediately
 * followed by a digit will return error and a value of 0, when
 * the next getc will return that following non-digit.  This is
 * caused by the single level ungetc available.
 */
bool readxint(int *val, FILE *f);

/*--------------------------------------------------------------
 * Flush input through an end-of-line marker inclusive.
 */
void flushln(FILE *f);

#endif
/* End of txtinput.h */
